@@ -40,6 +40,8 @@ char * fgetheader( FILE * f, const char * header ) {
 	    strcat( hdr , p );
 	} else if ( ( strncasecmp( p, header, hlen ) == 0 ) ) {
 	    p += hlen;
+	    if ( *p == ':' )
+	        p++;
 	    while ( p && *p && isspace((unsigned char)*p) )
 		p++;
 	    hdr = strdup( p );
