@@ -189,7 +189,8 @@ void delposted( void ) {
     a = posted;
     while ( a ) {
 	b = a;
-	unlink( a->string );
+	if ( unlink( a->string ) && ( errno != ENOENT ) )
+	    syslog( LOG_ERR, "Failed to unlink %s: %m", a->string );
 	a = a->next ;
 	free( b );
     }
@@ -659,7 +660,7 @@ int getgroup ( struct newsgroup * g, int server ) {
 	     ( now-st.st_ctime > (timeout_long * SECONDS_PER_DAY) ) ) {
 	    if ( verbose > 1 )
 		printf("skipping %s from now on\n", g->name);
-	    syslog( LOG_INFO, "skip %s (%d): %d, %d", g->name, now,
+	    syslog( LOG_INFO, "skip %s (%d): %d, %d", g->name, (int)now,
 		    now-st.st_ctime, now-st.st_mtime );
 	    unlink(s);
 	    g->first = 1;
@@ -720,7 +721,7 @@ int getgroup ( struct newsgroup * g, int server ) {
     }
 
     if ( outstanding == 0 ) {
-	return server;
+	return last+1;
     }
 
     syslog( LOG_INFO, "%s: will fetch %d articles", g->name, outstanding );
@@ -1072,9 +1073,10 @@ int getgroup ( struct newsgroup * g, int server ) {
 static void nntpactive( time_t update ) {
     struct stat st;
     char * l, * p;
-    struct stringlist * a, * b, *c = NULL;
+    struct stringlist * groups = NULL;
+    struct stringlist * helpptr = NULL;
     char timestr[64];
-    int reply = 0, error;
+    int reply = 0, error, merge = 0;
 
     sprintf( s, "%s/leaf.node/%s", spooldir, current_server->name );
     if ( active && !forceactive && ( stat( s, &st ) == 0 ) ) {
@@ -1088,34 +1090,31 @@ static void nntpactive( time_t update ) {
 	    syslog( LOG_ERR, "Reading new newsgroups failed" );
 	    return;
 	}
-	a = NULL;
 	while ( (l=getaline(nntpin)) && ( *l != '.' ) ) {
+	    merge++;
 	    p = l;
 	    while (!isspace((unsigned char)*p) )
 		p++;
 	    if (*p)
 		*p = '\0';
-	    b = (struct stringlist *)critmalloc( sizeof(struct stringlist)
-		 + strlen(l), "allocating space for newsgroup name");
-	    strcpy( b->string, l );
-	    b->next = NULL;
-	    if ( a == NULL )
-		a = b;
-	    else
-		c->next = b;
-	    c = b;
+	    insertgroup( l, 1, 1, time(NULL) );
+	    appendtolist( &groups, &helpptr, l );
 	}
 	if ( !l )		/* timeout */
 	    return;
-	b = a;
-	while ( b != NULL ) {
+	if ( merge ) {
+	    mergegroups();		/* merge groups into active */
+	    merge = 0;
+	}
+	helpptr = groups;
+	while ( helpptr != NULL ) {
 	    if ( current_server->descriptions ) {
 		error = 0;
-		sprintf( lineout, "XGTITLE %s\r\n", b->string );
+		sprintf( lineout, "XGTITLE %s\r\n", helpptr->string );
 		putaline();
 		reply = nntpreply();
 		if ( reply != 282 ) {
-		    sprintf( lineout, "LIST NEWSGROUPS %s\r\n", b->string );
+		    sprintf( lineout, "LIST NEWSGROUPS %s\r\n", helpptr->string );
 		    putaline();
 		    reply = nntpreply();
 		    if ( reply && ( reply != 215 ) )
@@ -1132,7 +1131,7 @@ static void nntpactive( time_t update ) {
 		    	    p++;
 			}
 			if ( reply == 215 || reply == 282 )
-			    insertgroup( l, 0, 0, *p ? p : NULL , update );
+			    changegroupdesc( l, *p ? p : NULL );
 			do {
 			    l = getaline(nntpin);
 			    error++;
@@ -1140,30 +1139,21 @@ static void nntpactive( time_t update ) {
 			if ( error > 1 ) {
 			    syslog( LOG_NOTICE, "warning: %s does not process "
 				   "LIST NEWSGROUPS %s correctly: use nodesc\n",
-				   current_server->name, b->string );
+				   current_server->name, helpptr->string );
 			    printf("warning: %s does not process LIST "
 				   "NEWSGROUPS %s correctly: use nodesc\n",
-				   current_server->name, b->string );
+				   current_server->name, helpptr->string );
 			}
 		    }
-		    else
-			insertgroup( b->string, 0, 0, NULL, update );
 		}
 	    } /* if ( current_server->descriptions ) */
-	    else
-		insertgroup( b->string, 0, 0, NULL, update );
-	    b = b->next;
-	}
-	b = a;
-	while ( b != NULL ) {
-	   c = b->next;
-	   free(b);
-	   b = c;
+	    helpptr = helpptr->next;
 	}
+	freelist( groups );
     } else {
 	if ( verbose )
 	    printf( "Getting all newsgroups from %s\n", current_server->name );
-	sprintf(lineout, "LIST ACTIVE\r\n" );
+	sprintf( lineout, "LIST ACTIVE\r\n" );
 	putaline();
 	if ( nntpreply() != 215 ) {
 	    printf( "Reading all newsgroups failed.\n" );
@@ -1179,33 +1169,36 @@ static void nntpactive( time_t update ) {
                 *p = '\0';
                 p++;
             }
-            insertgroup( l, 0, 0, NULL, update );
+            insertgroup( l, 1, 1, 0 );
         }
-	debug = debugmode;
+	mergegroups();
 	if ( !l )		/* timeout */
 	    return;
-	sprintf(lineout, "LIST NEWSGROUPS\r\n" );
+	sprintf( lineout, "LIST NEWSGROUPS\r\n" );
 	putaline();
 	l = getaline( nntpin );
 	/* correct reply starts with "215". However, INN 1.5.1 is broken
 	   and immediately returns the list of groups */
-	if ( ( l == NULL ) || ( strlen(l) < 4 ) ) {
+	if ( l ) {
+	    char *p;
+	    reply = strtol( l, &p, 10 );
+	    if ( ( reply == 215 ) && ( *p == ' ' || *p == '\0' ) )
+		l = getaline( nntpin ); /* get first description */
+	    else if ( *p != ' ' && *p != '\0' )
+		/* INN 1.5.1: line already contains description */
+		;
+	    else {
+		printf( "Reading newsgroups descriptions failed: %s.\n", l );
+		syslog( LOG_ERR, "Reading newsgroups descriptions failed: %s",
+			l );
+		return;
+	    }
+	}
+	else {
 	    printf( "Reading newsgroups descriptions failed.\n" );
 	    syslog( LOG_ERR, "Reading newsgroups descriptions failed" );
 	    return;
 	}
-	debug = 0;
-	if ( l[3] == ' ' ) {
-	    /* assume that a correct reply has the format "215 blablabla" */
-	    reply = strtol( l, NULL, 10 );
-	    if ( reply != 215 ) {
-		printf( "Reading newsgroups descriptions failed.\n" );
-		syslog( LOG_ERR, "Reading newsgroups descriptions failed: %d",
-			reply );
-		return;
-	    }
-	    l = getaline( nntpin );	/* get first description */
-	}
 	while ( l && ( strcmp( l, "." ) ) ) {
 	    p = l;
 	    while (!isspace((unsigned char)*p) )
@@ -1214,7 +1207,7 @@ static void nntpactive( time_t update ) {
                 *p = '\0';
                 p++;
             }
-	    insertgroup( l, 0, 0, *p ? p : NULL, update );
+	    changegroupdesc( l, *p ? p : NULL );
 	    l = getaline(nntpin);
 	}
 	debug = debugmode;
@@ -1451,12 +1444,12 @@ static int checkactive( void ) {
 	if ( debugmode )
 	    syslog( LOG_DEBUG,
 		    "Last LIST ACTIVE done %d seconds ago: NEWGROUPS\n",
-		    now - st.st_mtime );
+		    (int)(now - st.st_mtime) );
 	return st.st_atime ;
     } else {
 	if ( debugmode )
 	    syslog( LOG_DEBUG, "Last LIST ACTIVE done %d seconds ago: LIST\n",
-		    now - st.st_mtime );
+		    (int)(now - st.st_mtime) );
 	return 0;
     }
 }
@@ -1522,7 +1515,11 @@ int main( int argc, char ** argv ) {
     if ( !initvars( argv[0] ) )
 	exit( 1 );
 
+#ifdef HAVE_OLD_SYSLOG
+    openlog( "fetchnews", LOG_PID );
+#else
     openlog( "fetchnews", LOG_PID|LOG_CONS, LOG_NEWS );
+#endif
 
     while ( (option=getopt( argc, argv, "Pflnvx:" )) != -1 ) {
 	if ( option == 'v' ) {
@@ -1651,7 +1648,7 @@ int main( int argc, char ** argv ) {
 		syslog( LOG_DEBUG, "Process forked." );
 	    if ( !postonly ) {
 		updateactive();
-		writeactive( forceactive );
+		writeactive();
 		fixxover();
 		unlink( lockfile );
 	    }
