@@ -36,11 +36,12 @@ extern int errno;
 char * strdup (const char *);
 #endif
 
+#ifndef HAVE_SNPRINTF
+int snprintf( char *str, size_t n, const char *format, ... );
+#endif
+
 /* add LOG_NEWS where it doesn't exist */
 #include <syslog.h>
-#if !defined( LOG_NFACILITIES )
-#error "Leafnode does not compile with BSD 4.2 syslog"
-#endif
 #if !defined( LOG_NEWS )
 #define LOG_NEWS LOG_DAEMON
 #endif
@@ -75,8 +76,6 @@ int chdirgroup(const char *group, int creatdir);
  * newsgroup management
  */
 struct newsgroup {
-    struct newsgroup * left;
-    struct newsgroup * right;
     unsigned long first;
     unsigned long last;
     int alive;
@@ -86,11 +85,13 @@ struct newsgroup {
 };
 
 int isinteresting( const char * groupname );
-void insertgroup(const char * name, int first, int last, const char * desc,
-	int date );
+void insertgroup(const char * name, long unsigned first, long unsigned last,
+   int date );
+void changegroupdesc( const char * groupname, char * description );
+void mergegroups( void );
 struct newsgroup * findgroup( const char* name );
 void readactive( void );
-void writeactive( int forcedwrite );
+void writeactive( void );
 void fakeactive( void );
 
 extern struct newsgroup * active;
@@ -110,6 +111,9 @@ struct stringlist {
     struct stringlist * next;
     char string[1] ;
 };
+
+void appendtolist( struct stringlist ** list, struct stringlist ** lastentry,
+    char * newentry ) ;
 char * findinlist( struct stringlist * haystack, char * needle ) ;
 	/* find a string in a stringlist by doing a linear search */
 void freelist( struct stringlist * list ) ;
