@@ -234,8 +234,8 @@ void parser(void) {
 	fflush(stdout);
     }
     if ( debugmode )
-	syslog( LOG_DEBUG, ">421 Network error" );
-    error("421 Network error");
+	syslog( LOG_DEBUG, ">400 Service discontinued" );
+    error("400 Service discontinued");
 }
 
 /*
@@ -252,7 +252,7 @@ FILE * buildpseudoart(const char * grp) {
 	return f;
     }
 
-    fprintf(f, "Path: %s\n", fqdn);
+    fprintf(f, "Path: %s\n", owndn ? owndn : fqdn);
     fprintf(f, "Newsgroups: %s\n", grp);
     fprintf(f, "From: Leafnode <nobody@%s>\n", fqdn);
     fprintf(f, "Subject: Leafnode placeholder for group %s\n", grp);
@@ -269,7 +269,7 @@ FILE * buildpseudoart(const char * grp) {
 	  "will go and fetch articles from that group when it next updates.\n"
 	  "\n"
 	  "Since you have read this dummy article, leafnode will retrieve\n"
-	  "the newsgroup %s when fetch is run\n"
+	  "the newsgroup %s when fetchnews is run\n"
 	  "the next time. If you'll look into this group a little later, you\n"
 	  "will see real articles.\n"
 	  "\n"
@@ -280,7 +280,7 @@ FILE * buildpseudoart(const char * grp) {
 	  "If you do not understand this, please talk to your newsmaster.\n"
 	  "\n"
 	  "Leafnode can be found at\n"
-	  "\thttp://wpxx02.toxi.uni-wuerzburg.de/~krasel/leafnode.html\n\n", grp );
+	  "\thttp://www.leafnode.org/\n\n", grp );
 
     rewind(f);
     return f;
@@ -656,20 +656,19 @@ void domove( int by ) {
 
 /* LIST ACTIVE if what==0, else LIST NEWSGROUPS */
 void list(struct newsgroup *g, int what, const char * pattern) {
-    if (g) {
-	list (g->right, what, pattern);
+    while ( g->name ) {
 	if (what) {
 	    if ( !pattern )
-		printf( "%s\t%s\r\n", g->name, g->desc ? g->desc : "" );
+		printf( "%s\t%s\r\n", g->name, g->desc ? g->desc : "-x-" );
 	    else if ( ngmatch( pattern, g->name ) == 0 )
-		printf( "%s\t%s\r\n", g->name, g->desc ? g->desc : "" );
+		printf( "%s\t%s\r\n", g->name, g->desc ? g->desc : "-x-" );
 	} else {
 	    if ( !pattern )
 		printf( "%s %010lu %010lu y\r\n", g->name, g->last, g->first);
 	    else if ( ngmatch( pattern, g->name ) == 0 )
 		printf( "%s %010lu %010lu y\r\n", g->name, g->last, g->first);
 	}
-	list (g->left, what, pattern);
+	g++;
     }
 }
 	
@@ -751,18 +750,10 @@ void dolist( char *arg ) {
     }
 }
 
-static void helpdonewgroups( struct newsgroup *g, time_t age ) {
-    if ( g ) {
-	helpdonewgroups( g->right, age );
-	if ( g->age >= age )
-	    printf( "%s %lu %lu y\r\n", g->name, g->first, g->last );
-	helpdonewgroups( g->left, age );
-    }
-}
-
 void donewgroups(const char *arg) {
     struct tm timearray;
     struct tm * ltime;
+    struct newsgroup *g;
     time_t age;
     time_t now;
     int year, century;
@@ -801,12 +792,21 @@ void donewgroups(const char *arg) {
     timearray.tm_hour = a / 10000;
     timearray.tm_min  = a % 10000 / 100;
     timearray.tm_sec  = a % 100;
-    age = mktime( &timearray ) ;
+#ifdef HAVE_GMTOFF
+    age = mktime( &timearray ) + (strstr( arg, "GMT" ) ? 0 : ltime->tm_gmtoff );
+#else
+    age = mktime( &timearray ) - (strstr( arg, "GMT" ) ? 0 : timezone);
+#endif
 
     printf( "231 List of new newsgroups follows\r\n" );
     if ( debugmode )
 	syslog( LOG_DEBUG, ">231 List of new newsgroups follows" );
-    helpdonewgroups( active, age );
+    g = active;
+    while ( g->name ) {
+	if ( g->age >= age )
+	    printf( "%s %lu %lu y\r\n", g->name, g->first, g->last );
+	g++;
+    }
     printf( ".\r\n" );
 }
 
@@ -952,7 +952,10 @@ void dopost( void ) {
 	else {
 	    if ( !havepath ) {
 		write( out, "Path: ", 6 );
-		write( out, fqdn, strlen( fqdn ) );
+		if ( owndn )
+		    write( out, owndn, strlen( owndn ) );
+		else
+		    write( out, fqdn, strlen( fqdn ) );
 		write( out, "!nobody\r\n", 9 );
 	    }
 	    if ( !havedate ) {
@@ -1023,8 +1026,8 @@ void doxhdr(char *arg) {
     int n = 7;
     int dash;
     char * l;
+    char * buf;
     int a,b=0,c;
-    FILE * f;
 
     if ( !arg || !*arg ) {
 	if ( debugmode )
@@ -1043,6 +1046,7 @@ void doxhdr(char *arg) {
 	*l++ = '\0';
     while( l && *l && isspace((unsigned char)*l) )
 	l++;
+    buf = strdup(arg);
 
     if (l && *l=='<') { /* handle message-id form (well) */
 	FILE * f;
@@ -1057,20 +1061,21 @@ void doxhdr(char *arg) {
 	do {
 	    l = getaline( f );
 	} while (l && *l && 
-		 strncasecmp(arg, l, strlen(arg)) && 
-		 l[strlen(arg)]!=':'); /* uh, space before : allowed? */
+		 strncasecmp(buf, l, strlen(buf)) && 
+		 l[strlen(buf)]!=':'); /* uh, space before : allowed? */
 	while( l && *l && !isspace((unsigned char)*l) )
 	    l++;
 	while( l && *l && isspace((unsigned char)*l) )
 	    l++;
 	if ( debugmode ) {
-	    syslog( LOG_DEBUG, ">221 %s header follows:", arg );
+	    syslog( LOG_DEBUG, ">221 %s header follows:", buf );
 	    syslog( LOG_DEBUG, ">%s", l ? l : "" );
 	    syslog( LOG_DEBUG, "." );
 	}
 	printf( "221 %s header follows:\r\n%s\r\n.\r\n",
-		arg, l ? l : "" ); /* dubious - if not found, "" */
+		buf, l ? l : "(none)" );
 	fclose( f );
+	free(buf);
 	return;
     }
 
@@ -1096,17 +1101,17 @@ void doxhdr(char *arg) {
     if ( pseudogroup ) {
 	do {
 	    n--;
-	} while ( n >= 0 && strncasecmp( h[n], arg, strlen(h[n]) ) != 0 ) ;
+	} while ( n >= 0 && strncasecmp( h[n], buf, strlen(h[n]) ) != 0 ) ;
 	if ( n < 0 ) {
-	    printf( "430 No such header: %s\r\n", arg );
+	    printf( "430 No such header: %s\r\n", buf );
 	    if ( debugmode )
-		syslog( LOG_DEBUG, ">430 No such header: %s", arg );
+		syslog( LOG_DEBUG, ">430 No such header: %s", buf );
 	    return;
 	}
 	if ( debugmode )
 	    syslog( LOG_DEBUG,
-		    ">221 First line of %s pseudo-header follows:", arg );
-	printf( "221 First line of %s pseudo-header follows:\r\n", arg );
+		    ">221 First line of %s pseudo-header follows:", buf );
+	printf( "221 First line of %s pseudo-header follows:\r\n", buf );
 	if ( n == 0 )		/* Subject: */
 	    printf( "1 Leafnode placeholder for group %s\r\n", group->name );
 	else if ( n == 1 )	/* From: */
@@ -1116,12 +1121,13 @@ void doxhdr(char *arg) {
 	else if ( n == 3 )	/* Message-ID: */
 	    printf( "1 <leafnode:placeholder:%s@%s>\r\n", group->name, fqdn );
 	else if ( n == 4 )	/* References */
-	    printf( "1\r\n" );
+	    printf( "1 (none)\r\n" );
 	else if ( n == 5 )	/* Bytes */
 	    printf( "1 %d\r\n", 1024 );	/* just a guess */
 	else if ( n == 6 )	/* Lines */
 	    printf( "1 %d\r\n", 22 );		/* from buildpseudoart() */
 	 printf( ".\r\n" );
+	 free( buf );
 	 return;
     }
 
@@ -1159,7 +1165,7 @@ void doxhdr(char *arg) {
 
     do {
 	n--;
-    } while( n>-1 && strncasecmp(arg, h[n], strlen(h[n])) );
+    } while( n>-1 && strncasecmp(buf, h[n], strlen(h[n])) );
 
     if ( a < group->first )
 	a = group->first;
@@ -1194,32 +1200,23 @@ void doxhdr(char *arg) {
 		    if (p)
 			*p = '\0';
 		}
-		printf("%d %s\r\n", c, l && *l ? s : "");
+		printf("%d %s\r\n", c, l && *l ? s : "(none)");
 	    }
 	}
     } else {
 	if ( debugmode )
-	    syslog( LOG_DEBUG, ">221 %s header (from overview) "
-		    "for postings %d-%d:",arg,a,b );
+	    syslog( LOG_DEBUG, ">221 %s header (from article files) "
+		    "for postings %d-%d:",buf,a,b );
 	printf( "221 %s header (from article files) for postings %d-%d:\r\n",
-		arg, a, b );
-	n = strlen( arg );
+		buf, a, b );
 	for( c=a; c<=b; c++ ) {
 	    sprintf( s, "%d", c );
-	    f = fopen( s, "r");
-	    while( f ) {
-		l = getaline( f );
-		if ( !l || !*l || *l == '\r' || *l == '\n' ) {
-		    fclose( f );
-		    f = 0;
-		} else if ( l[n] == ':' &&
-			    strncasecmp( arg, l, n ) == 0 ) {
-		    printf("%d %s\r\n", c, l + n + 2 );
-		}
-	    }
+	    l = getheader( s, buf );
+	    printf("%d %s\r\n", c, l ); /* (l && *l) ? l : "(none)" ); */
 	}
     }
 
+    free(buf);
     printf( ".\r\n" );
     return;
 }
@@ -1416,7 +1413,11 @@ int main( int argc, char ** argv ) {
     verbose = 0;
     umask(2);
 
+#ifdef HAVE_OLD_SYSLOG
+    openlog( "leafnode", LOG_PID );
+#else
     openlog( "leafnode", LOG_PID|LOG_CONS, LOG_NEWS );
+#endif
 
     printf("200 Leafnode NNTP Daemon, version %s running at %s\r\n", 
 	   version, fqdn);
