@@ -324,10 +324,10 @@ static void dogroup(struct newsgroup* g) {
 	}
     }
 
-    if (last < first) {
-	closedir(d);
+    closedir( d );
+    if (last < first)
 	return;
-    }
+
     if ( verbose > 1 )
         printf("%s: low water mark %lu, high water mark %lu\n",
 	        g->name, first, last );
@@ -339,8 +339,6 @@ static void dogroup(struct newsgroup* g) {
     if ( expire <= 0 )
 	return;
 
-    rewinddir( d );
-
     /* allocate and clear article array */
 
     articles = (struct exp*)critmalloc((last - first + 1) * sizeof(struct exp),
@@ -399,10 +397,14 @@ static void dogroup(struct newsgroup* g) {
     }
 
     /* insert articles in tree, and clear 'kill' for new or read articles */
-
+    d = opendir(".");
+    if (!d) {
+	syslog(LOG_ERR, "opendir in %s: %m", gdir);
+	return;
+    }
     while ((de = readdir(d)) != 0) {
 	art = strtoul(de->d_name, &p, 10);
-	if (p && !*p) {
+	if (p && !*p && art <= last && art >= first) {
 	    articles[art - first].exists = 1;
 	    /* mark all articles as to-be-deleted and rescue those
 	     * which fulfill certain criteria
@@ -509,12 +511,11 @@ static void dogroup(struct newsgroup* g) {
 
 static void expiregroup(struct newsgroup* g)
 {
-    if ( g ) {
-	expiregroup( g->right );
+    while ( g && g->name ) {
 	if (!(expire = lookup_expire(g->name)))
 	    expire = default_expire;
 	dogroup( g );
-	expiregroup( g->left );
+	g++;
     }
 }
 
@@ -568,7 +569,11 @@ int main(int argc, char** argv)
     if ( !initvars( argv[0] ) )
 	exit( 1 );
 
+#ifdef HAVE_OLD_SYSLOG
+    openlog( "texpire", LOG_PID );
+#else
     openlog( "texpire", LOG_PID|LOG_CONS, LOG_NEWS );
+#endif
     while ( (option=getopt( argc, argv, "vf" )) != -1 ) {
 	if ( option == 'v' ) {
             verbose++;
@@ -616,7 +621,7 @@ int main(int argc, char** argv)
     default_expire = expire;
 
     expiregroup( active );
-    writeactive( 0 );
+    writeactive();
     unlink( lockfile );
     expiremsgid();
     free_expire();
